#/* Pdx/Docbook.py 
# * Copyright (C) 1999  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */

__version__='0.4'

import sys,os,time,re,string
import Base

#revisions:
#1999-09-24  Harry George    reimplement from perl version

def debug(ftn,txt):
    sys.stdout.write("Pdx.Docbook."+ftn+":"+txt+"\n")
    sys.stdout.flush()

class Docbook (Base.Base):
    classname="Docbook"
    def __init__(self):
        Base.Base.__init__(self)
        self.driver="docbook"
        self.extension=".sgml"
        self.data['_section_level']=0
        self.data['_in_section_p']=0
        self.data['_in_para_p']=0
    #------------------
    def __del__(self):
        Base.Base.__del__(self)
    #--------------------
    def driver_reset(self):
        self.data['_section_level']=0
        self.data['_in_section_p']=0
        self.data['_in_para_p']=0        
    #--------------------
    def driver_center(self):
        self.driver_para_end()
        self.rawout('<BLOCKQUOTE>')
        self.driver_para()
        self.process_file()
        self.nextline()  #past the =end
        self.driver_para_end()
        self.rawout('</BLOCKQUOTE>')
        self.driver_para()
    #--------------------
    def driver_head(self,label,level,name):
        ftn="Docbook.driver_head"
        if self._pass==1:
            data=[level,name,label]
            self._toc.append(data)
        level=int(level)
        self.driver_head_end(level)
        self.data['_section_level']=level
        self.rawout("<SECT%i><TITLE>" % level)
        self.out(name)
        self.rawout("</TITLE>\n")
        self.driver_para()
    #--------------------
    def driver_head_end(self,level=0):
        ftn="driver_head_end"
        self.driver_para_end()
        currlevel=int(self.data['_section_level'])
        while currlevel > 0 and currlevel >= level:
            self.rawout("</SECT%i>\n" % currlevel)
            currlevel=currlevel-1

    #--------------------
    def driver_head_numbered(self,label,level,name):
        ftn="driver_head_numbered"
        self.driver_head(label,level,name)
    #--------------------
    def driver_index(self):
        if self.data['index_p']==0: return
        if len(self._index)==0: return
        self._index.sort()
        self.rawout('\n<INDEX><TITLE>Index</TITLE>\n');
        currterm=''
        term=label=None
        for data in self._index:
            term,label=string.split(data,':')
            if term != currterm:
                currterm=term
                self.rawout('<INDEXENTRY><PRIMARYIE '
                 + 'LINKENDS="' + label + '">' 
                 + currterm + '</PRIMARYIE></INDEXENTRY>\n');

        self.rawout("</INDEX>\n")
            
    #--------------------
    def driver_line(self,line):
        ftn = "driver_line"
        #---text markups---
        line=self.hr_pat.sub('',line)
        line=self.break_pat.sub('',line)
        line=self.B_pat.sub('<EMPHASIS>\\1</EMPHASIS>',line)
        line=self.I_pat.sub('<EMPHASIS>\\1</EMPHASIS>',line)
        line=self.C_pat.sub('<LITERAL>\\1</LITERAL>',line)
        line=self.U_pat.sub('<EMPHASIS>\\1</EMPHASIS>',line)
        #---links---
        line=self.N_pat.sub('<ANCHOR ID="\\1">',line)
        line=self.L_hash_pat.sub('<LINK ID="\\1">\\2</LINK>',line)
        line=self.L_pat.sub('<ULINK URL="\\1">\\2</ULINK>',line)
        #---escapes---
        line=self.lt_pat.sub('&lt;',line)
        line=self.gt_pat.sub('&gt;',line)
        line=self.amp_pat.sub('&amp;',line)
        line=self.quot_pat.sub('&quot;',line)
        line=self.lb_pat.sub('#',line)

        #debug(ftn,"cooked line="+line)
        return line
        
    #--------------------
    def driver_list(self,marker):
        ftn = "driver_list"
        #debug(ftn,"begin with "+marker)
        self.driver_para_end()
        before_map={
            '*':'<ITEMIZEDLIST MARK="bullet">',
            's':'<ITEMIZEDLIST MARK="square">',
            'c':'<ITEMIZEDLIST MARK="circle">',
            'N':'<ORDEREDLIST NUMERATION="Arabic" INHERITNUM="INHERIT">',
            '1':'<ORDEREDLIST NUMERATION="Arabic">',
            'a':'<ORDEREDLIST NUMERATION="loweralpha">',
            'A':'<ORDEREDLIST NUMERATION="upperalpha">',
            'i':'<ORDEREDLIST NUMERATION="lowerroman">',
            'I':'<ORDEREDLIST NUMERATION="upperroman">',
            '[':'<GLOSSLIST>'
            }
        after_map={
            '*':'</ITEMIZEDLIST>',
            's':'</ITEMIZEDLIST>',
            'c':'</ITEMIZEDLIST>',
            'N':'</ORDEREDLIST>',
            '1':'</ORDEREDLIST>',
            'a':'</ORDEREDLIST>',
            'A':'</ORDEREDLIST>',
            'i':'</ORDEREDLIST>',
            'I':'</ORDEREDLIST>',
            '[':'</GLOSSLIST>',
            }
        before=before_map[marker]
        after=after_map[marker]
        #debug(ftn,"before="+before+", after="+after)
        in_item_p=0
        
        self.rawout(before)
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line) :
                if in_item_p:
                    self.driver_list_item_end(marker)
                break
            if self.back_pat.search(line):
                self.driver_list_item_end(marker)
                break
            m=self.item_pat.search(line)
            if m:
                if in_item_p:
                    self.driver_list_item_end(marker)
                in_item_p=1
                txt=m.group(1)
                self.driver_list_item_begin(marker,txt)
            else:
                self.ndx=self.ndx-1
                self.process_file()
        self.rawout(after)
        
    #--------------------
    def driver_list_item_begin(self,marker,txt):
        if marker == '[':
            self.out('<GLOSSENTRY><GLOSSTERM>%s</GLOSSTERM>\n' % (txt,))
            self.rawout("<GLOSSDEF><PARA>\n")
        else:
            self.out('\n<LISTITEM><PARA> %s\n' % (txt,))
        
    #--------------------
    def driver_list_item_end(self,marker):
        if marker == '[':
            self.rawout("</PARA></GLOSSDEF></GLOSSENTRY>\n")
        else:
            self.rawout("</PARA></LISTITEM>\n")    
    #-------------------
    def driver_para(self):
        ftn = "driver_para"
        self.driver_para_end()
        self.rawout("\n<PARA>")
        self.data['_in_para_p']=1

    #--------------------
    def driver_para_end(self,level=1):
        if self.data['_in_para_p']==1:
            self.rawout("</PARA>\n")
            self.data['_in_para_p']=0
##         level=int(level)
##         sectlevel=self.data['_section_level']
##         while level < sectlevel:
##             #self.rawout("</SECT"+`sectlevel`+">")
##             self.rawout("</SECTION>")
##             sectlevel=sectlevel-1
##         self.data['_section_level']=sectlevel
        
    #--------------------
    def driver_quote(self):
        self.driver_para_end()
        self.rawout('<BLOCKQUOTE>')
        self.driver_para()
        self.process_file()
        self.nextline()  #past the =end
        self.driver_para_end()
        self.rawout('</BLOCKQUOTE>')
        self.driver_para()
    #--------------------
    def driver_table_begin(self,caption):
        widths = self.table_colwidths
        numcols= self.table_numcols
        self.rawout('<TABLE ')
        if self.table_grid_p:
            self.rawout('FRAME="ALL" COLSEP="1" ROWSEP="1">')
        else:
            self.rawout('FRAME="NONE" COLSEP="0" ROWSEP="0">')
        self.rawout('\n<TITLE>%s</TITLE>'% caption)

        self.rawout('\n<TGROUP COLS="%i">' % numcols)
        if widths:
            for i in range(numcols):
                width=widths[i]
                self.rawout('\n<COLSPEC COLNUM="%i" COLWIDTH="%i*">' % \
                        (i+1,width))
        self.rawout('\n<TBODY>\n')
    #--------------------
    def driver_table_end(self,caption):
        self.rawout('</TBODY></TGROUP></TABLE>\n')

    #--------------------
    def driver_table_row_begin(self):
        self.rawout('<ROW>')
    #--------------------
    def driver_table_row_end(self):
        self.rawout('</ROW>\n')
    #--------------------
    def driver_table_cell_begin(self,width=None):
        self.rawout('<ENTRY><PARA>')
    #--------------------
    def driver_table_cell_end(self):
        self.rawout('</PARA></ENTRY>\n')
    #--------------------
    def driver_texmath(self,lines):
        name=self.data['_basename'] + '_' + self.genlabel()
        self.do_texmath_png(name,lines)
        self.rawout('<GRAPHIC FILEREF="'+name+'.png" FORMAT="'+name+'.png">')
    #--------------------
    def driver_toc(self):
        ftn="driver_toc"
        #debug(ftn,"begin, toc="+`self._toc`)
        self.rawout('<TOC><TOCCHAP>\n')
        self.rawout('<TOCENTRY>Table Of Contents</TOCENTRY>\n')
        currlevel=0
        level=name=label=None
        for entry in self._toc:
            level,name,label=entry
            level=int(level)
            #--go to right level---
            if currlevel==level:
                pass
            elif currlevel > level:
                while 1:
                    if currlevel > 0 and currlevel > level:
                        self.rawout("</TOCLEVEL%i>\n" % currlevel)
                        currlevel=currlevel-1
                    else:
                        currlevel=level
                        #2002-06-08 HGG:  HACK.  This looks wrong,
                        #but it passes the docbook parser.
                        self.rawout("<TOCLEVEL%i>\n" % currlevel)
                        break
            elif currlevel < level:
                while 1:
                    if currlevel < level:
                        currlevel=currlevel+1
                        self.rawout("<TOCLEVEL%i>\n" % currlevel)
                    else:
                        break
            #---new entry---
            self.out("<TOCENTRY>%s</TOCENTRY>\n" % name)
        while currlevel > 0:
            self.rawout("</TOCLEVEL%i>" % currlevel)
            currlevel=currlevel-1
        self.rawout('</TOCCHAP></TOC>\n')

    #--------------------
    def driver_verbatim(self,lines):
        self.rawout("\n"+'<PROGRAMLISTING>'+"\n")
        for line in lines:
            line=line.replace('<','&lt;')
            line=line.replace('>','&gt;')
            self.rawout(line)
        self.rawout('</PROGRAMLISTING>'+"\n")
    #---------------------
    def driver_xref(self,term):
        ftn = "do_xref"
        label=self.genlabel()
        if self._pass==1:
            data=string.join([term,label],":")
            self._index.append(data)
            self._refs[term]=label
        xref_txt = '<INDEXTERM ID="'+label+'"><PRIMARY>'+term+'</PRIMARY></INDEXTERM>'
        return xref_txt

#==main=======================

if __name__=='__main__':
   ftn="main"


