#/* Pdx/Latex.py 
# * Copyright (C) 2000  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */

__version__='0.4'

import sys,os,time,re,string
import Base

#revisions:
#2000-01-09  Harry George    reimplement from perl version

def debug(ftn,txt):
    sys.stdout.write("Pdx.Latex."+ftn+":"+txt+"\n")
    sys.stdout.flush()

#-------------------------------------------------
class  Latex(Base.Base):
    classname="Latex"
    def __init__(self):
        Base.Base.__init__(self)
        self.driver="latex"
        self.extension=".tex"
    #------------------
    def __del__(self):
        Base.Base.__del__(self)
    #--------------------
    def driver_center(self):
        self.rawout('\\begin{center}')
        self.process_file()
        self.nextline()  #past the =end
        self.rawout(r'\end{center}')
    #--------------------
    def driver_head_marker(self,level):
        doc_class_to_offset = {
            "article" : 1,
            "book"    : 0,
            "report"  : 0,
            "memo"    : 2,
            "seminar" : 2,
            }
        pdx_to_tex_head = {
            1 : "chapter",
            2 : "section",
            3 : "subsection",
            4 : "subsubsection",
            5 : "paragraph",
            }

        docclass=self.data['docclass']
        offset=doc_class_to_offset[docclass]
        return pdx_to_tex_head[int(level)+offset]

    #--------------------
    def driver_head(self,label,level,name):
        global get_head_level,pdx_to_tex_head
        if self._pass==1:
            data=[level,name,label]
            self._toc.append(data)
        marker=self.driver_head_marker(level)
        self.rawout('\\%s*{' % marker)
        self.out(name)
        self.rawout('}\n')

    #--------------------
    def driver_head_numbered(self,label,level,name):
        marker=self.driver_head_marker(level)
        self.rawout('\\%s{' % marker)
        self.out(name)
        self.rawout('}\n')
    #--------------------
    def driver_index(self):
        self.rawout('\\printindex\n')
            
    #--------------------
    def driver_line(self,line):
        ftn = "driver_line"
        #---text markups---
        line=self.hr_pat.sub(r'\\newpage',line)
        line=self.break_pat.sub(r'\\\\',line)
        line=self.B_pat.sub(r'{\\bfseries '+ '\\1}',line)
        line=self.I_pat.sub(r'{\\itshape '+ '\\1}',line)
        line=self.C_pat.sub(r'{\\ttfamily '+ '\\1}',line)
        line=self.U_pat.sub(r'\\underline{'+'\\1}',line)

        #---escape all '_', then undo url's---
        line=line.replace('_','\\_')
        
        #---latex and pdflatex treat links differently---
        pdf_p=self.data.get('pdf_p',None)
        m=self.N_pat.search(line)
        if m:
            url=m.group(1)
            url=url.replace('\\_','-') #_' illegal in labels, so use '-'
            if pdf_p:
                line=self.N_pat.sub('\\hypertarget{%s}{ }' % url,line)
            else:
                line=self.N_pat.sub(r'\\label{%s}' % url,line)
                

        m=self.L_hash_pat.search(line)
        if m:
            url=m.group(1)
            url=url.replace('\\_','-') #'_' illegal in labels, so use '-'
            if pdf_p:
                line=self.L_hash_pat.sub('\\hyperlink{%s}{\\2}' % url,line)
            else:
                line=self.L_hash_pat.sub('\\2 [pg \\pageref{%s}]' % url,line)
                

        m=self.L_pat.search(line)
        if m:
            url=m.group(1)
            if pdf_p:
                url=url.replace('\\_','_') #undo escape
                #---check for leading url type, use "file:" as default---
                if url.find('http:')==0 or url.find('ftp:')==0 or \
                   url.find('file:')==0:
                    line=self.L_pat.sub('\\href{%s}{\\2}' % url,line)
                else:
                    line=self.L_pat.sub('\\href{file:./%s}{\\2}' % url,line)
            else:
                line=self.L_pat.sub('\\2 [%s]' % url,line)

        #---escapes---
        line=self.lt_pat.sub(r'$<$',line)
        line=self.gt_pat.sub(r'$>$',line)
        line=self.amp_pat.sub(r'\&',line)
        line=self.quot_pat.sub(r'"',line)
        line=self.lb_pat.sub(r'\#',line)

        #debug(ftn,"cooked line="+line)
        return line
        
    #--------------------
    def driver_list(self,marker):
        ftn = "driver_list"
        #debug(ftn,"begin with "+marker)
        counter=self.genlabel()
        before_map={
            '*':r'\begin{itemize}',
            's':r'\begin{itemize}',
            'c':r'\begin{itemize}',
            'N':r'\begin{itemize}',
            '1':r'\begin{enumerate}',
            'a':
        r'\newcounter{'+counter+'} \\begin{list}{\\alph{'+counter+"}.}\n"
      + r'{\usecounter{'+counter+"}\n"
      + r'\setlength{\labelwidth}{5ex}'
      + r'\setlength{\parsep}{0.5ex plus0.2ex minus0.1ex}'
      + r'\setlength{\itemsep}{0ex plus0.2ex minus0.1ex}}' 
            ,'A':
        r'\newcounter{'+counter+'} \\begin{list}{\\Alph{'+counter+"}.}\n"
      + r'{\usecounter{'+counter+"}\n"
      + r'\setlength{\labelwidth}{5ex}'
      + r'\setlength{\parsep}{0.5ex plus0.2ex minus0.1ex}'
      + r'\setlength{\itemsep}{0ex plus0.2ex minus0.1ex}}' 
            ,'[':'\\begin{description}',
            'i':r'\begin{enumerate}[i]',
            'I':r'\begin{enumerate}[I]',

            }
        after_map={
            '*':r'\end{itemize}',
            's':r'\end{itemize}',
            'c':r'\end{itemize}',
            'N':r'\end{itemize}',
            '1':r'\end{enumerate}',
            'a':r'\end{list}',
            'A':r'\end{list}',
            'i':r'\end{enumerate}',
            'I':r'\end{enumerate}',
            '[':r'\end{description}',
            }
        before=before_map[marker]
        after=after_map[marker]
        #debug(ftn,"before="+before+", after="+after)
        self.rawout(before)

        lists=self.liststack
        if marker=='N':
            #---fully numbered list, starting at 0---
            lists.append(0)
            numbered_p=1
        else:
            numbered_p=0

        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line) : break  #normal lists
            if self.back_pat.search(line): break  #pod-style over...back
            m=self.item_pat.search(line)
            if m:
                txt=m.group(1)
                if numbered_p:
                    lists[-1]=lists[-1]+1
                    numstr=''
                    for item in lists:
                        numstr=numstr+`item`+'.'
                    self.out('\n\\item[%s] %s\n' % (numstr,txt))
                elif marker == '[':
                    self.out('\n\\item[%s]\n' % (txt,))
                else:
                    self.out('\n\\item %s\n' % (txt,))
            else:
                self.ndx=self.ndx-1
                self.process_file()
        self.rawout(after)
        if numbered_p:
            lists.pop()
                    
    #-------------------
    def driver_para(self):
        ftn = "driver_para"
        #debug(ftn,"begin")
        self.rawout("\n")

    #--------------------
    def driver_quote(self):
        self.rawout('\\begin{quote}')
        self.process_file()
        self.nextline()  #past the =end
        self.rawout(r'\end{quote}')
    #--------------------
    def driver_table_begin(self,caption):
        ftn = "driver_table_begin"
        numcols=self.table_numcols
        widths =self.table_colwidths
        self.rawout('\\begin{tabular}{')
        #---col format string---
        if self.table_grid_p: self.rawout('|')
        for i in range(numcols):
            if widths:
                fraction=0.01*widths[i]
                self.rawout('p{%2.2f\\textwidth} ' % fraction)
            else:
                self.rawout('l')
            if self.table_grid_p: self.rawout('|')
        
        self.rawout('}')

        if self.table_grid_p: self.rawout('\\hline ')
        self.rawout("\n")
        
##     #--------------------
##     def driver_table_row(self,cells):
##         numcols=self.table_numcols
##         widths = self.table_colwidths
##         for i in range(numcols):
##             cell = cells[i]
##             if i!=0 : self.rawout(' & ')
##             if widths:
##                 fraction=0.01*widths[i]
##                 self.out('\\begin{minipage}[t]'
##                             + r'{' + `fraction` + r'\textwidth} '
##                             + cell
##                             + r' \end{minipage}')
##             else:
##                 self.out(cell)
##         self.rawout(r'\tabularnewline ')
##         if self.table_grid_p: self.rawout(r'\hline ')
##         self.rawout("\n")
        
    #--------------------
    def driver_table_row_begin(self):
        pass
    #--------------------
    def driver_table_row_end(self):
        self.rawout(r'\tabularnewline ')
        if self.table_grid_p: self.rawout(r'\hline ')
        self.rawout("\n")
    #--------------------
    def driver_table_cell_begin(self,width=None):
        if self.table_col_ndx > 0:
            self.rawout(' & ')
        if self.table_colwidths:
            fraction=0.01*width
            self.rawout('\\begin{minipage}[t]{%2.2f\\textwidth}' % fraction)
    #--------------------
    def driver_table_cell_end(self):
        if self.table_colwidths:
            self.out(r'\end{minipage}')

    #--------------------
    def driver_table_end(self,caption):
        self.rawout('\end{tabular}'+"\n")
    #--------------------
    def driver_texmath(self,lines):
        for line in lines:
          self.rawout(line)  
    #--------------------
    def driver_toc(self):pass
    #--------------------
    def driver_verbatim(self,lines):
        self.rawout("\n"+'\\begin{verbatim}'+"\n")
        for line in lines:
            line=self.lt_pat.sub('&lt;',line)
            line=self.gt_pat.sub('&gt;',line)
            line=self.amp_pat.sub('&amp;',line)
            line=self.quot_pat.sub('&quot;',line)
            self.rawout(line)
        self.rawout(r'\end{verbatim}'+"\n")

    #---------------------
    def driver_xref(self,term):
        ftn = "do_xref"
        return r'\index{'+term+r'}'

#==main=======================
def test(basename,show):
    ftn="test"
    sys.stderr.write(basename+"...")
    olddir=os.getcwd()
    os.chdir('../test')
    pdxname=basename+'.pdx'
    texname=basename+'.tex'
    dviname=basename+'.dvi'
    idxname=basename+'.idx'
    p=Latex()
    p.parse([pdxname])
    sys.stderr.write(p.data['desc']+"...")
    ndx=basename[-2:]
    testname="test"+ndx+".tex"
    oraclename="oracle"+ndx+".tex"
    result=commands.getoutput("diff "+testname +" "+oraclename)
    if len(result)==0:
        sys.stderr.write("passed\n")
    else:
        sys.stderr.write("failed:\n"+result+"\n")

    if show:
        os.system("latex "+texname)
        os.system("makeindex "+idxname)
        os.system("latex "+texname)
        os.system("xdvi "+dviname)
    os.chdir(olddir)

if __name__=='__main__':
   ftn="main"
   import commands
   debug(ftn,"testing")
   show=0
   test("test01",show)
   test("test02",show)
   test("test03",show)
   test("test04",show)
   test("test05",show)
   test("test06",show)
   test("test07",show)
   test("test08",show)
   test("test10",show)

