#/* Pdx/Rest.py 
# * Copyright (C) 1999  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */

__version__='0.1'

import sys,os,time,re,string,cgi
import Base
import tabbedwriter

#revisions:
#2012-01-07   Harry George    Adding ReST

def debug(ftn,txt):
    sys.stdout.write("Pdx.Rest."+ftn+":"+txt+"\n")
    sys.stdout.flush()


#section markers starting with None because level is 1-based
section_markers=[None,'=','-','~','^','_','<','>']

transition_marker='------------'


#====================================
class Rest (Base.Base):
    classname="Rest"
    #------------------
    def __init__(self):
        Base.Base.__init__(self)
        self.driver="rest"
        self.extension=".rst"
        self.table_row_fmt=''
        self.table_dashline=''
        
    #---------------------
    # For indents, need tabbedwriter
    #---------------------
    def open_files(self):
        Base.Base.open_files(self)
        #---need tabbedwriter----
        f=self.driverfile
        wr=tabbedwriter.TabbedWriter(f)
        self.driverfile=wr
    #---------------------
    def indent(self):
        self.driverfile.indent()
    #---------------------
    def undent(self):
        self.driverfile.undent()
    #---------------------
    def nodent(self):
        self.driverfile.tabs=0
        self.driverfile.pad=''
    #---------------------
    def rawout(self,line,nl=1,pad=1):
        if self._pass==1:
            return
        self.driverfile.write(line,nl=nl,pad=pad)
    #--------------------
    #--------------------
    def __del__(self):
        Base.Base.__del__(self)
    #--------------------
    #--------------------
    def driver_center(self):
        self.indent()
        self.indent()
        self.process_file()
        self.nextline()  #past the =end
        self.undent()
        self.undent()
        self.rawout("")
    #--------------------
    def driver_head(self,label,level,name):
        if self._pass==1:
            data=[level,name,label]
            self._toc.append(data)
            return

        self.nodent()
        marker=section_markers[int(level)]
        header='_`%s` %s' % (label,name)
        cnt=len(header)+4
        dashline=cnt*marker

        self.rawout(dashline)
        self.out(header)
        self.rawout(dashline)
    #--------------------
    def driver_head_numbered(self,label,level,name):
        """
        Given stack of headings with [[1],[1,1],[1,2],[2]]
        get the current head, and build the label.
        Maintain stack.
        """        
        ftn="driver_head_numbered"
        level=int(level)
        heads=self.headstack
        
        #---fix the stack---
        while 1:
            if len(heads)==0:
                #debug(ftn,"fresh stack")
                heads.append([1])
                break
            
            top=heads[-1]
            toplevel=len(top)
            #debug(ftn,"level=%i,heads=%s" % (level,`heads`))
            
            if level==toplevel:
                #debug(ftn,"new head at same level")
                num=top[-1]
                num=num+1
                top[-1]=num
                heads[-1]=top
                break
            
            elif level < toplevel:
                #debug(ftn,"pop to parent,then add")
                while level < len(heads[-1]):
                    heads.pop()
            
            elif level > toplevel:
                #debug(ftn,"add a child")
                newtop=[]
                for x in top:
                    newtop.append(x)
                newtop.append(1)
                heads.append(newtop)
                break
            
        #---build the new label---
        newtop=heads[-1]
        label=""
        for x in newtop:
            label=label+`x`+'.'

        #---report result----
        header='_`%s` %s' % (label,name)
        marker=section_markers[int(level)]
        cnt=len(header)+4
        dashline=cnt*marker

        if self._pass==1:
            #---append to table of contents---
            data1=[level,name,label]
            self._toc.append(data1)
            return
        
        self.rawout(dashline)
        self.out(header)
        self.rawout(dashline)
            
    #--------------------
    def driver_index(self):
        ftn="driver_index"
        #debug(ftn,"begin with index="+`self._index`)
        if len(self._index)==0: return
        self._index.sort()
        self.do_head("1","Index")
        self.indent()
        currterm=''
        term=label=None
        for data in self._index:
            term,label=string.split(data,':')
            if term != currterm:
                currterm=term
                self.out("- %s `%s`_" % (currterm,label))
        self.undent()
        self.rawout("")
            
    #--------------------
    def driver_line(self,line):
        ftn = "driver_line"
        #---text markups---
        line=self.B_pat.sub('**\\1**',line)
        line=self.I_pat.sub('*\\1*',line)
        line=self.C_pat.sub('``\\1``',line)
        line=self.U_pat.sub('___\\1___',line)
        #---breaks---
        line=self.hr_pat.sub(transition_marker,line)      
        line=self.break_pat.sub('\n\n'+self.driverfile.pad,line)
        #---links---
        line=self.N_pat.sub('_`\\1`',line)      
        line=self.L_hash_pat.sub('`\\1`_: \\2',line)
        line=self.L_pat.sub('`\\2 <\\1>`_ ',line)
        #---escapes---
        line=self.lt_pat.sub('<',line)
        line=self.gt_pat.sub('>',line)
        line=self.amp_pat.sub('&',line)
        line=self.quot_pat.sub('"',line)
        line=self.lb_pat.sub('#',line)
        line=self.vertbar_pat.sub('\|',line)

        #debug(ftn,"cooked line="+line)
        return line

    #--------------------
    def driver_list(self,marker):
        ftn = "driver_list"
        item_map={
            '*':'-',
            's':'+',  #"square"
            'c':'*',  #"circle"
            'N':'1.',
            '1':'1.',
            'a':'a)',
            'A':'A.',
            'i':'i',   #roman numeral
            'I':'I',   #Roman Numeral
            '[':'',
            }
        item_marker=item_map[marker]
        
        lists=self.liststack
        if marker=='N':
            #---fully numbered list, starting at 0---
            lists.append(0)
            numbered_p=1
        else:
            numbered_p=0
            
        first_item_p=True
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line) :
                self.undent()
                self.rawout("")
                break
            
            if self.back_pat.search(line):
                self.rawout("")
                break

            m=self.item_pat.search(line)
            if m:
                txt=m.group(1)

                #---indented, numbered---
                if numbered_p:
                    if first_item_p:
                        self.indent()
                        first_item_p=False
                        
                    lists[-1]=lists[-1]+1
                    numstr=''
                    for item in lists:
                        numstr=numstr+`item`+'.'
                    self.out('- %s %s' % (numstr,txt))
                    
                #---definitions---
                elif marker == '[':
                    self.nodent()
                    self.out('%s' % (txt,))
                    self.indent()

                #---bullets---
                elif marker=='*':
                    if first_item_p:
                        self.indent()
                        first_item_p=False
                    self.out('%s %s' % (item_marker,txt,))

                #---enumerations---
                else:
                    if first_item_p:
                        self.indent()
                        first_item_p=False
                        tmp=item_marker
                    else:
                        tmp='#.'
                    self.out('%s %s' % (tmp,txt,))
            else:
                self.ndx=self.ndx-1
                self.indent()
                self.process_file()
                self.undent()
                #self.rawout("")
        if numbered_p:
            lists.pop()
    #-------------------
    def driver_para(self):
        ftn = "driver_para"
        #debug(ftn,"begin")
        self.rawout("\n")
    #--------------------
    def driver_quote(self):
        self.indent()
        self.rawout('')
        self.process_file()
        self.nextline()  #past the =end
        self.undent()
    #-----------------------
    # table 
    #-----------------------
    #--------------------
    def driver_table_begin(self,caption):
        """Override the work in Base"""
        ftn = "driver_table_begin"
        cells=self.table_cells
        #debug(ftn,"cells=%s" % `cells`)
        
        numcols=self.table_numcols=len(cells)
        widths =self.table_colwidths=[]
        fmt=self.table_row_fmt=''
        dashline=self.table_dashline=''

        for cell in cells:
            w=len(cell)+4
            widths.append(w)
            fmt+='%%-%i.%is ' % (w,w) 
            dashline+='='*w+' '

        #---done---
        self.table_row_fmt=fmt
        self.table_dashline=dashline
        #debug(ftn,"numcols=%i" % self.table_numcols) 
        #debug(ftn,"full fmt= |%s|" % self.table_row_fmt) 
        #debug(ftn,"full dash=|%s|" % self.table_dashline) 
        
    #--------------------
    def do_table_cells(self,cells):
        """override Base"""
        ftn='do_table_cells'
        c=tuple(cells)
        #debug(ftn,"c=%s" % `c`)
        fmt=self.table_row_fmt
        #debug(ftn,"fmt= |%s|" % fmt)
        dashline=self.table_dashline
        #debug(ftn,"dash=|%s|" % dashline)
        try:
            self.out(fmt % c)
        except:
            debug(ftn,"c=%s failed" % `c`)

    #--------------------
    def driver_table_row_begin(self):
        if self.first_row_p:
            self.rawout(self.table_dashline)
            
    #--------------------
    def driver_table_row_end(self):
        if self.first_row_p:
            self.rawout(self.table_dashline)

    #--------------------
    def driver_table_cell_begin(self,width=None):
        pass
    #--------------------
    def driver_table_cell_end(self):
        pass

    #--------------------
    def driver_table_end(self,caption):
        self.rawout(self.table_dashline)
        
    #--------------------
    # end table
    #--------------------
    def driver_texmath(self,lines):
        name=self.data['_basename'] + '_' + self.genlabel()
        self.do_texmath_png(name,lines)
        self.rawout('.. image:: %s.png\n' % name)
    #--------------------
    def driver_toc(self):
        ftn = "driver_toc"
        self.nodent()
        name='Table of Contents'
        cnt=len(name)+2
        marker=section_markers[1]
        dashline=cnt*marker
        
        self.rawout(dashline)
        self.out(name)
        self.rawout(dashline)
        self.rawout("")

        currlevel=1
        level=name=label=None
        for entry in self._toc:
            level,name,label=entry
            level=int(level)
            while currlevel < level:
                self.indent()
                currlevel+=1
            while currlevel > level:
                self.undent()
                currlevel-=1
            self.out("- `%s`_ %s" % (label,name))
        while currlevel > 0:
            self.undent()
            currlevel-=1
            
    #-------------------------
    def driver_verbatim(self,lines):
        ftn="driver_verbatim"
        self.nodent()
        self.rawout("::\n")
        self.indent()
        for line in lines:
            line=cgi.escape(line,quote=1)
            self.rawout(line)
        self.nodent()
        
    #---------------------
    def driver_xref(self,term):
        ftn = "do_xref"
        label=self.genlabel()
        if self._pass==1:
            data=string.join([term,label],":")
            self._index.append(data)
            self._refs[term]=label
        xref_txt = '_`%s` ' % label
        return xref_txt

#==main=======================
def test(basename):
    ftn="test"
    sys.stderr.write(basename+"...")
    olddir=os.getcwd()
    os.chdir('../test')
    p=Rest()
    pdxname=basename+'.pdx'
    p.parse([pdxname])
    sys.stderr.write(p.data['desc']+"...")
    ndx=basename[-2:]
    testname="test"+ndx+".rest"
    oraclename="oracle"+ndx+".rest"
    result=commands.getoutput("diff "+testname +" "+oraclename)
    resultlen=len(result)
    if resultlen==0:
        sys.stderr.write("passed\n")
    else:
        sys.stderr.write("failed:\n"+result+"\n")
    os.chdir(olddir)
    del p

def testloop(basename):
    olddir=os.getcwd()
    os.chdir('testdata')
    pdxname=basename+'.pdx'
    for i in range(1,10):
        p=Rest()
        p.parse([pdxname]) 
        del p
    os.chdir(olddir)

if __name__=='__main__':
   ftn="main"
   import commands,profile
   #---get local Base and Rest---
   del Base
   sys.path.insert(0,'.')
   sys.path.insert(0,'..')
   import Base

   #profile.run('testloop("test01")')
   test("test01")
   test("test02")
   test("test03")
   test("test04")
   test("test05")
   test("test06")
   test("test07")
   test("test08")
   test("test10")

