#!/usr/bin/env python

"""
NAME:
    model.py  

SYNOPSIS:
    model.py [options] filename

DESCRIPTION:
    Model (in the sense of model-view-controller) used by Pdx.

OPTIONS:
    -h,--help        this message
    -v,--version     version
    -d,--debug       turn on debug messages
    --bb=xyz         set option bb to xyz

EXAMPLES:
    1. As standalone
        model.py -d myfile
    2. As library
        import model
        ...

#Copyright (C) 2003  Harry George hgg9140@seanet.com
#http://www.seanet.com/~hgg9140
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


"""
#===imports======================
import sys,os,getopt,time

#===globals======================
modname="model"
__version__="0.1"

#--option args--
debug_p=0
#opt_b=None  #string arg, default is undefined

#---positional args, default is empty---
pargs=[]    

#---other---

#===utilities====================
def msg(txt):
    sys.stdout.write(txt)
    sys.stdout.flush()

def debug(ftn,txt):
    if debug_p:
        sys.stdout.write("%s.%s:%s\n" % (modname,ftn,txt))
        sys.stdout.flush()

def fatal(ftn,txt):
    msg="%s.%s:FATAL:%s\n" % (modname,ftn,txt)
    raise SystemExit, msg
 
def usage():
    print __doc__
#====================================
# Styles
#====================================
class ParaStyle:
    left=0
    center=1
    right=2
    
    #---class variables---
    #--------------------------
    def __init__(self,name=""):
        self.name=name
        self.parent=None
        self.align=ParaStyle.left

#====================================
class TextStyle:
    #---class variables---
    #--------------------------
    def __init__(self,
                 name="",
                 bold=False,
                 italics=False,
                 underline=False,
                 strikethrough=False,
                 font='arial',
                 fontsize=12):
        self.name=name
        self.bold=bold
        self.italics=italics
        self.underline=underline
        self.strikethrough=strikethrough
        self.font=font
        self.fontsize=fontsize


#====================================
# Structures
#====================================
class DocumentType:
    unknown=0
    book=1
    report=2
    article=3
    
class Document:
    #---class variables---
    #--------------------------
    def __init__(self,typ=DocumentType.unknown):
        ftn="Document.__init__"
        self.typ=typ
        self.sections=[]
        self.glossary={}
        self.index={}
        #---metadata---
        self.title=''
        self.authors=[]
        self.date=''
        self.docid=''
        self.source=''
        
#====================================
class SectionType:
    normal=1
    abstract=2
    appendix=3

class Section:
    #---class variables---
    #--------------------------
    def __init__(self,typ=SectionType.normal,level=0):
        ftn="Section.__init__"
        self.typ=typ
        self.level=level
        self.children=[]
        
#==========================        
class Para:
    #---class variables---
    #--------------------------
    def __init__(self):
        self.style="Normal"
        self.children=[]

#==========================
class Text:
    #---class variables---
    #--------------------------
    def __init__(self):
        self.style="Normal"
        self.children=[]
        
#==========================
# Lists
#==========================
class ListType:
    bullet=0
    arabic=1
    roman=2
    
    
class ListElement:
    #---class variables---
    #--------------------------
    def __init__(self):
        self.tag=None
        self.body=None
        
class List:
    #---class variables---
    #--------------------------
    def __init__(self,typ=ListType.bullet):
        self.typ=typ
        self.children=[]  #elements
        
#==============================
# Tables
#==============================
class Row:
    #---class variables---
    #--------------------------
    def __init__(self):
        #---instance variables---
        self.cells=[]
        
class Table:
    #---class variables---
    #--------------------------
    def __init__(self):
        #---instance variables---
        self.caption=""
        self.header_rows=[]
        self.grid="" # ""=none, "V"=vert, "H"=horiz, "B"=both
        self.rows=[]
        
#==============================
# Markups
#==============================
class Hyperlink:
    def __init__(self):
        #---instance variables---
        self.url=""
        self.text=""

class Image:
    def __init__(self):
        #---instance variables---
        self.imagefile=""
        self.temp_text=""
        self.caption=""

class Xref:
    def __init__(self):
        #---instance variables---
        self.target=""
        self.text=""

class Index:
    def __init__(self):
        #---instance variables---
        self.entry=""
        self.text=""

        
        
    
#====================================
# Main model
#====================================
class Model:
    #---class variables---
    #--------------------------
    def __init__(self):
        ftn="Model.__init__"
        #---instance variables---
        (self.year,self.month,self.day,self.hour,self.minute,self.second,
         self.weekday,self.julianday,self.daylight)=time.localtime(time.time())
        self.body=None

        self.para_styles={}
        self.para_styles['Normal']=ParaStyle(name='Normal')

        self.text_styles={}
        self.text_styles['Normal']=TextStyle(name='Normal')
        
        self.preamble_begin=[]
        self.preamble_append=[]
        self.preamble_end=[]
        self.postamble=[]


    #--------------------------
#=============================
def main(args):
    x=Model()
    x.run()
    
#-------------------------
if __name__ == '__main__':
    ftn = "main"
    opts,pargs=getopt.getopt(sys.argv[1:],'hvd',
                 ['help','version','debug',
                  'bb='])
    for opt in opts:
        if opt[0]=='-h' or opt[0]=='--help':
            print modname+": version="+__version__
            usage()
            sys.exit(0)
        elif opt[0]=='-v' or opt[0]=='--version':
            print modname+": version="+__version__
            sys.exit(0)
        elif opt[0]=='-d' or opt[0]=='--debug':
            debug_p=1
        elif opt[0]=='--bb':
            opt_b=opt[1]

    #---make the object and run it---
    main(pargs)

#===Revision Log===
#Created by mkpythonproj:
#2003-11-23  Harry George  
#
